% Derman et al static hedging of barrier options.
% Replicate the barrier call option payoff at the barrier,
% using vanilla calls only

clc; clear;
S = 100;    % Stock price
H = 120;    % Barrier
K = 100;    % Strike Price
r = 0.05;   % Risk free rate
q = 0.02;   % Dividend Yield
v = 0.15;   % Volatility

% Inline function for the Black Scholes call price
BSCall = @(S,K,r,q,T,vol) S*exp(-q*T)*normcdf((log(S/K) + (r-q+vol^2/2)*T)/vol/sqrt(T)) - K*exp(-r*T)*normcdf((log(S/K) + (r-q+vol^2/2)*T)/vol/sqrt(T) - vol*sqrt(T));

% One call with strike = K and two calls with strike = H
t = [1; 0.5; 0];                % Time steps
a =  alpha(H, K, t, r, q, v)';  % Find the required number of calls (weights)
a = flipud(a);
Time2 = [t(1):-.01/2:0];        % Time increments must fall on the time steps
t = flipud(t);

for i=1:length(Time2);
	Price2(i,1) = BSCall(H,K,r,q,max(Time2(i),1e-15),v);
	for j=1:length(t)-1;
		Price2(i,j+1) = BSCall(H,H,r,q,max(Time2(i)-t(j),1e-15),v)*a(j);
		if isnan(Price2(i,j+1))
			Price2(i,j+1)=0;
		end
	end
end
P2 = sum(Price2');              % Portfolio is the sum of the weighted options


% One call with strike = K and three calls with strike = H
clear a t
t = [1; 2/3; 1/3; 0];
a =  alpha(H, K, t, r, q, v)';
a = flipud(a);
Time3 = [t(1):-.01/3:0];
t = flipud(t);

for i=1:length(Time3);
	Price3(i,1) = BSCall(H,K,r,q,max(Time3(i),1e-15),v);
	for j=1:length(t)-1;
		Price3(i,j+1) = BSCall(H,H,r,q,max(Time3(i)-t(j),1e-15),v)*a(j);
		if isnan(Price3(i,j+1))
			Price3(i,j+1)=0;
		end
	end
end
P3 = sum(Price3');

% One call with strike = K and six calls with strike = H
clear a t;
t = [12; 10; 8; 6; 4; 2; 0]./12;
a =  alpha(H, K, t, r, q, v)';
a = flipud(a);
Time6 = [t(1):-.01/3:0];
t = flipud(t);

for i=1:length(Time6);
	Price6(i,1) = BSCall(H,K,r,q,max(Time6(i),1e-15),v);
	for j=1:length(t)-1;
		Price6(i,j+1) = BSCall(H,H,r,q,max(Time6(i)-t(j),1e-15),v)*a(j);
		if isnan(Price6(i,j+1))
			Price6(i,j+1)=0;
		end
	end
end
P6 = sum(Price6');

% Plot the results
plot(Time2', P2','k-.', Time3', P3', 'k--', Time6', P6', 'k-')
set(gca,'XDir','reverse')
xlabel('Portfolio value along the barrier');
ylabel('Time to expiry');
legend('One call with same strike as barrier call + 2 calls with strike = barrier',...
       'One call with same strike as barrier call + 3 calls with strike = barrier',...
       'One call with same strike as barrier call + 6 calls with strike = barrier',...
	   'Location', 'NorthWest');

